#include <QtGui/QApplication>
#include <QTranslator>
#include <QLocale>
#include <QStringBuilder>
#include <QTextCodec>

#include "frmmain.h"
#include "version.h"

/** @brief Интернет-домен организации - разработчика программы */
#define MY_DOMAIN_ABBR "life-warrior.org"

// Константы параметров командной строки

/** @brief Параметр командной строки, означающий текущую версию программы, наличие обновлений для которой надо проверить. */
#define CMDPAR_VER_CUR "curver="
/** @brief Длинна строки с параметром командной строки, означающий текущую версию программы, наличие обновлений для которой надо проверить. */
#define PARLEN_VER_CUR 7
/** @brief Параметр командной строки, означающий адрес страницы сайта, с которой можно получить файл с номером текущей версии программы. */
#define CMDPAR_SITE_VER "site="
/** @brief Длинна строки с параметром командной строки, означающий адрес страницы сайта, с которой можно получить файл с номером текущей версии программы. */
#define PARLEN_SITE_VER 5
/** @brief Параметр командной строки, означающий адрес страницы сайта, переход на которую надо предложить пользователю в случае наличия новой версии программы. */
#define CMDPAR_SITE_GOTO "goto="
/** @brief Длинна строки с параметром командной строки, означающий адрес страницы сайта, переход на которую надо предложить пользователю в случае наличия новой версии программы. */
#define PARLEN_SITE_GOTO 5
/** @brief Параметр командной строки, означающий название программы, обновления для которой надо проверять. */
#define CMDPAR_PROG "prog="
/** @brief Длинна строки с параметром командной строки, означающий название программы, обновления для которой надо проверять. */
#define PARLEN_PROG 5
/** @brief Параметр командной строки, означающий режим отображения сообщения об отсутствии новых версий. */
#define CMDPAR_NOUP_MGS "nupmsg="
/** @brief Длинна строки с параметром командной строки, означающим режим отображения сообщения об отсутствии новых версий. */
#define PARLEN_NOUP_MGS 7

int main(int argc, char *argv[])
{
    // Переменные.
    // Адрес сайта, по которому надо проверять новую версию программы:
    QString sPageForCheck;
    // Флаг наличия параметра командной строки с адресом сайта, по которому надо
    // проверять новую версию программы:
    bool bPageForCheck = false;
    // Текущая версия программы:
    QString sCurVer;
    // Флаг наличия параметра командной строки с текущей версией программы:
    bool bCurVer = false;
    // Адрес сайта, по которому надо переходить в случае обнаружения новой версии
    // программы:
    QString sPageGoTo;
    // Флаг наличия параметра командной строки с адресом сайта, по которому надо
    // переходить в случае обнаружения новой версии программы:
    bool bPageGoTo = false;
    // Название проверяемой программы:
    QString sProg;
    // Идентификатор режима отображения сообщений об отсутствии обновлений:
    int iNupMsg = 0;
    // Перебор параметров командной строки.
    for (int i=0; i<argc; ++i)
    {
        QString sNextPar = QString(argv[i]);
        if (sNextPar.startsWith(CMDPAR_VER_CUR, Qt::CaseInsensitive))
        {
            // Убираем название параметра и знак "=":
            sNextPar.remove(0, PARLEN_VER_CUR);
            // Сохраняем версию в переменной:
            sCurVer = sNextPar;
            // Проверяем, не пустой ли параметр. Если не пустой -
            // поднимаем флаг его наличия:
            bCurVer = !sCurVer.isEmpty();
        }
        else if (sNextPar.startsWith(CMDPAR_PROG, Qt::CaseInsensitive))
        {
            // Убираем название параметра и знак "=":
            sNextPar.remove(0, PARLEN_PROG);
            // Сохраняем название программы:
            sProg = sNextPar;
        }
        else if (sNextPar.startsWith(CMDPAR_SITE_VER, Qt::CaseInsensitive))
        {
            // Убираем название параметра и знак "=":
            sNextPar.remove(0, PARLEN_SITE_VER);
            // Сохраняем адрес сайта для проверки версии в переменной:
            sPageForCheck = sNextPar;
            // Проверяем, не пустой ли параметр. Если не пустой -
            // поднимаем флаг его наличия:
            bPageForCheck = !sPageForCheck.isEmpty();
        }
        else if (sNextPar.startsWith(CMDPAR_SITE_GOTO, Qt::CaseInsensitive))
        {
            // Убираем название параметра и знак "=":
            sNextPar.remove(0, PARLEN_SITE_GOTO);
            // Сохраняем адрес сайта для проверки версии в переменной:
            sPageGoTo = sNextPar;
            // Проверяем, не пустой ли параметр. Если не пустой -
            // поднимаем флаг его наличия:
            bPageGoTo = !sPageGoTo.isEmpty();
        }
        else if (sNextPar.startsWith(CMDPAR_NOUP_MGS, Qt::CaseInsensitive))
        {
            // Убираем название параметра и знак "=":
            sNextPar.remove(0, PARLEN_NOUP_MGS);
            // Пробуем перевести строку в число:
            iNupMsg = sNextPar.toInt();
        }
    }

    // Создание приложения:
    QApplication a(argc, argv);
    // Устанавливаем данные приложения:
    a.setApplicationName(APP_NAME_SIGN);
    a.setOrganizationName(MY_DOMAIN_ABBR);
    a.setOrganizationDomain(MY_DOMAIN_ABBR);
    // Загрузка справки для приложения.
    // Устанавливаем кодек для переводов:
    QTextCodec::setCodecForTr(QTextCodec::codecForName(UTF8_ABBR));
    // Создаём объект переводчика:
    QTranslator snTranslator;
    // В случае, если успешно загрузится файл перевода - устанавливаем
    // переводчика:
    if(snTranslator.load("LWUpChecker_" % QLocale::system().name(),
        QApplication::applicationDirPath() % "/lang"))
        a.installTranslator(&snTranslator);


    // Создание главной формы приложения:
    frmMain w;

    // Параметры командной строки разобраны. Теперь надо проверить, есть ли все
    // необходимые параметры. Если они есть - проверить версию, если параметров
    // недостаточно - запустить программу в режиме редактирования настроек.
    if (bCurVer && bPageForCheck && bPageGoTo)
        w.checkSiteVer(sCurVer, sProg, sPageForCheck, sPageGoTo,  true,
            iNupMsg);
    else
        // Показываем главную форму:
        w.show();
    // Запускаем приложение:
    return a.exec();
}

